/**************************
* Includes
*
**************************/

#include <windows.h>
#include <gl/gl.h>
#include <gl/glu.h>
#include <time.h>
#include "gd.h"
#include "gettexture.h"
#include "reel.h"
#include "texfont.h"
#include "slotmachine.h"

/**************************
* Function Declarations
*
**************************/

LRESULT CALLBACK WndProc (HWND hWnd, UINT message,
WPARAM wParam, LPARAM lParam);
void EnableOpenGL (HWND hWnd, HDC *hDC, HGLRC *hRC);
void DisableOpenGL (HWND hWnd, HDC hDC, HGLRC hRC);

SlotMachine *slotmachine;

void CALLBACK cbFunct(UINT , UINT , DWORD_PTR , DWORD_PTR , DWORD_PTR) 
{ 
    slotmachine->physicsproc();
} 


void fullscreen(void)
{                
    DEVMODE dmScreenSettings;
    memset(&dmScreenSettings,0,sizeof(dmScreenSettings));
    dmScreenSettings.dmSize=sizeof(dmScreenSettings);
    dmScreenSettings.dmPelsWidth	= 640;
    dmScreenSettings.dmPelsHeight	= 480;
    dmScreenSettings.dmBitsPerPel	= 16;	
    dmScreenSettings.dmFields=DM_BITSPERPEL|
    DM_PELSWIDTH | DM_PELSHEIGHT;
    
    if (ChangeDisplaySettings(&dmScreenSettings,
    CDS_FULLSCREEN)!=DISP_CHANGE_SUCCESSFUL)
    MessageBox(NULL, "Screen mode error", "Error", MB_OK);    
    ShowCursor(FALSE);	    
}

/**************************
* WinMain
*
**************************/

int WINAPI WinMain (HINSTANCE hInstance,
HINSTANCE hPrevInstance,
LPSTR lpCmdLine,
int iCmdShow)
{
    WNDCLASS wc;
    HWND hWnd;
    HDC hDC;
    HGLRC hRC;        
    MSG msg;
    BOOL bQuit = FALSE;
    
    /* register window class */
    wc.style = CS_OWNDC;
    wc.lpfnWndProc = WndProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = 0;
    wc.hInstance = hInstance;
    wc.hIcon = LoadIcon (NULL, IDI_APPLICATION);
    wc.hCursor = LoadCursor (NULL, IDC_ARROW);
    wc.hbrBackground = (HBRUSH) GetStockObject (BLACK_BRUSH);
    wc.lpszMenuName = NULL;
    wc.lpszClassName = "GLSample";
    RegisterClass (&wc);            
    
    //fullscreen();

    int screenwidth = GetSystemMetrics(SM_CXSCREEN);
    int screenheight = GetSystemMetrics(SM_CYSCREEN);
                    
    /* create main window */
    hWnd = CreateWindow (
    "GLSample", "Slot Machine", 
    WS_POPUP|WS_VISIBLE,
    0, 0, screenwidth, screenheight,
    NULL, NULL, hInstance, NULL);
    //    WS_CAPTION | WS_POPUPWINDOW | WS_VISIBLE
    /* enable OpenGL for the window */
    EnableOpenGL (hWnd, &hDC, &hRC);
    
    slotmachine = new SlotMachine();    

    timeSetEvent(25, 0, cbFunct, NULL, TIME_PERIODIC);   
    
    srand(time(NULL));
    
    glEnable(GL_LIGHTING);
    GLfloat LightAmbient[]= { .001f, .001f, .001f, 1.0f };
    GLfloat LightDiffuse[]= { .04f, .04f, .04f, 1.0f };
    GLfloat LightPosition[]= { 0.0f, 0.0f, -100.0f, 1.0f };
    glLightfv(GL_LIGHT1, GL_AMBIENT, LightAmbient);	
    glLightfv(GL_LIGHT1, GL_DIFFUSE, LightDiffuse);	
    glLightfv(GL_LIGHT1, GL_POSITION,LightPosition);
    glEnable(GL_LIGHT1);
    
    /* program main loop */
    while (!bQuit)
    {
        /* check for messages */
        if (PeekMessage (&msg, NULL, 0, 0, PM_REMOVE))
        {
            /* handle or dispatch messages */
            if (msg.message == WM_QUIT)
            {
                bQuit = TRUE;
            }
            else
            {
                TranslateMessage (&msg);
                DispatchMessage (&msg);
            }
        }
        else
        {
            /* OpenGL animation code goes here */
            
            glClearColor (0.0f, 0.0f, 0.0f, 0.0f);
            glLoadIdentity();
            gluPerspective(60.0f, 1.33, .1, 100.0f); 
            slotmachine->draw();        
                        
            SwapBuffers (hDC);                                               
            Sleep (1);
            
        }
    }
    
    delete slotmachine;
    
    /* shutdown OpenGL */
    DisableOpenGL (hWnd, hDC, hRC);
    
    /* destroy the window explicitly */
    DestroyWindow (hWnd);
    
    return msg.wParam;
}


/********************
* Window Procedure
*
********************/

LRESULT CALLBACK WndProc (HWND hWnd, UINT message,
WPARAM wParam, LPARAM lParam)
{
    
    switch (message)
    {
        case WM_CREATE:
        return 0;
        case WM_CLOSE:
        PostQuitMessage (0);
        return 0;
        
        case WM_DESTROY:
        return 0;
        
        case WM_KEYDOWN:
        
        switch (wParam)
        {            
            case VK_ESCAPE:
            PostQuitMessage(0);
            break;
            default:
            slotmachine->roll();                      
            return 0;
        }
        return 0;
        
        default:
        return DefWindowProc (hWnd, message, wParam, lParam);
    }
}


/*******************
* Enable OpenGL
*
*******************/

void EnableOpenGL (HWND hWnd, HDC *hDC, HGLRC *hRC)
{
    PIXELFORMATDESCRIPTOR pfd;
    int iFormat;
    
    /* get the device context (DC) */
    *hDC = GetDC (hWnd);
    
    /* set the pixel format for the DC */
    ZeroMemory (&pfd, sizeof (pfd));
    pfd.nSize = sizeof (pfd);
    pfd.nVersion = 1;
    pfd.dwFlags = PFD_DRAW_TO_WINDOW | 
    PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER;
    pfd.iPixelType = PFD_TYPE_RGBA;
    pfd.cColorBits = 24;
    pfd.cDepthBits = 16;
    pfd.iLayerType = PFD_MAIN_PLANE;
    iFormat = ChoosePixelFormat (*hDC, &pfd);
    SetPixelFormat (*hDC, iFormat, &pfd);
    
    /* create and enable the render context (RC) */
    *hRC = wglCreateContext( *hDC );
    wglMakeCurrent( *hDC, *hRC );
    
}


/******************
* Disable OpenGL
*
******************/

void DisableOpenGL (HWND hWnd, HDC hDC, HGLRC hRC)
{
    wglMakeCurrent (NULL, NULL);
    wglDeleteContext (hRC);
    ReleaseDC (hWnd, hDC);
}
